﻿

/**
* Prototypování
*/
function executeExample1() {

    // Jednoduchý objekt "kočka"
    function Cat(){
    }

    Cat.prototype.isMammal = function () {
        return true;
    };
    
    var myCat = new Cat();
    console.log("Moje kočka:", myCat);
    console.log("Je moje kočka savec:", myCat.isMammal());
    console.log("Typ objektu kočka:", typeof myCat);
    console.log("Je kočka instancí objektu 'Cat':", myCat instanceof Cat);
    console.log("Je kočka instancí objektu 'Object':", myCat instanceof Object);
    console.log("Je 'Object' instancí objektu kočka:", {} instanceof Cat);




    // Oddělovač
    console.log("\n\n\n");




    // Složitější objekt
    function Animal(isMamal) {
        this._isMamal = isMamal;
    }

    Animal.prototype.name = null;

    Animal.prototype.isMammal = function () {
        return this._isMamal;
    };

    Animal.prototype.getName = function () {
        return this.name;
    };

    Animal.prototype.sayHello = function () {
        return "Hi, I'm " + this.getName();
    };

    var dog = new Animal(true);
    dog.name = "Azor";
    var eagle = new Animal(false);
    eagle.name = "Golden Eagle";

    console.log("Je " + dog.name + " savec:", dog.isMammal());
    console.log("Je " + eagle.name + " savec:", eagle.isMammal());
    console.log("Pes:" + dog.sayHello());
    console.log("Orel: ", eagle.sayHello());

}





/**
* Funkce
*/
function executeExample2() {

    // Jednoduchý objekt "kočka"
    function Cat() {
        this.isMammal = function () {
            return true;
        };
    }

    var myCat = new Cat();
    console.log("Moje kočka:", myCat);
    console.log("Je moje kočka savec:", myCat.isMammal());
    console.log("Typ objektu kočka:", typeof myCat);
    console.log("Je kočka instancí objektu 'Cat':", myCat instanceof Cat);
    console.log("Je kočka instancí objektu 'Object':", myCat instanceof Object);
    console.log("Je 'Object' instancí objektu kočka:", {} instanceof Cat);




    // Oddělovač
    console.log("\n\n\n");




    // Složitější objekt
    function Animal(isMamal) {

        // NOTE: Proměnná isMammal je privátní

        this.name = null;

        this.isMammal = function () {
            return isMamal;
        };

        this.getName = function () {
            return this.name;
        };

        this.sayHello = function () {
            return "Hi, I'm " + this.getName();
        };
    }


    var dog = new Animal(true);
    dog.name = "Azor";
    var eagle = new Animal(false);
    eagle.name = "Golden Eagle";

    console.log("Je " + dog.name + " savec:", dog.isMammal());
    console.log("Je " + eagle.name + " savec:", eagle.isMammal());
    console.log("Pes:" + dog.sayHello());
    console.log("Orel: ", eagle.sayHello());

}





/**
* Objekt
*/
function executeExample3() {

    // Jednoduchý objekt "kočka"
    var Cat = function () {
        return {
            isMammal: function () {
                return true;
            }
        };
    }

    var myCat = new Cat();
    console.log("Moje kočka:", myCat);
    console.log("Je moje kočka savec:", myCat.isMammal());
    console.log("Typ objektu kočka:", typeof myCat);
    console.log("Je kočka instancí objektu 'Cat':", myCat instanceof Cat);
    console.log("Je kočka instancí objektu 'Object':", myCat instanceof Object);
    console.log("Je 'Object' instancí objektu kočka:", {} instanceof Cat);



    // Oddělovač
    console.log("\n\n\n");



    // Složitější objekt
    var Animal = function (isMammal) {

        // NOTE: Proměnná isMammal je privátní

        return {
            name: null,
            isMammal: function () {
                return isMammal;
            },
            getName: function () {
                return this.name;
            },
            sayHello: function () {
                return "Hi, I'm " + this.getName();
            }
        };
    };

    var dog = new Animal(true);
    dog.name = "Azor";
    var eagle = new Animal(false);
    eagle.name = "Golden Eagle";

    console.log("Je " + dog.name + " savec:", dog.isMammal());
    console.log("Je " + eagle.name + " savec:", eagle.isMammal());
    console.log("Pes:" + dog.sayHello());
    console.log("Orel: ", eagle.sayHello());

}












/**
* This
*/
function executeExample4()
{

    function Cat() {
        var a = 5;
        this.b = 10;
        this.c = 33;
    };

    Cat.prototype.getA = function () {
        return this.a;
    };

    Cat.prototype.getB = function () {
        return this.b;
    };

    Cat.prototype.getC = function () {
        return this.c;
    };

    Cat.prototype.getThis = function () {
        return this;
    };

    Cat.prototype.nastedFunction1 = function (c) {
        function setC(c) {
            this.c = c;
        }
        setC(c);
    };

    Cat.prototype.nastedFunction2 = function (c) {
        var _this = this;
        function setC(c) {
            _this.c = c;
        }
        setC(c);
    };
    
    Cat.prototype.nastedFunction3 = function (c) {
        function setC() {
            return this;
        }
        return setC();
    };

    Cat.prototype.nastedFunction4 = function (c) {
        var _this = this;
        function setC() {
            return _this;
        }
        return setC();
    };

    var myCat = new Cat();

    console.log("Hodnota proměnné 'a':", myCat.getA());
    console.log("Hodnota proměnné 'b':", myCat.getB());
    console.log("Hodnota proměnné 'c':", myCat.getC());

    console.log("this:", myCat.getThis());
    console.log("this == myCat:", myCat.getThis() == myCat);


    // Nastavíme 'c' pomocí zanořené funkce
    myCat.nastedFunction1(88);
    console.log("Hodnota 'c' po volání 'nastedFunction1':", myCat.getC());

    // Nastavíme 'c' pomocí zanořené funkce
    myCat.nastedFunction2(88);
    console.log("Hodnota 'c' po volání 'nastedFunction2':", myCat.getC());

    // this zahnízdělných metod
    console.log("this volání metody 'nastedFunction3':", myCat.nastedFunction3());
    console.log("this volání metody 'nastedFunction4':", myCat.nastedFunction4());

}








/**
* Dědičnost
*/
function executeExample5()
{
    // Pomocná metoda pro dědění
    function extend(aFn, bFn) {
        aFn.prototype = new bFn();
        aFn.prototype._base = bFn;
    }

    // Zvíře
    function Animal(isMammal) {
        this._isMammal = isMammal;
    }
    Animal.prototype.isMammal = function () {
        return this._isMammal;
    };


    // Kočka
    function Cat() {
        this._base(true);
    }
    // Kočka dědí ze zvířete
    extend(Cat, Animal);

    Cat.prototype.getNumberOfLegs = function () {
        return 4;
    };
   

    var myCat = new Cat();
    console.log("Moje kočka:", myCat);
    console.log("Typ objektu kočka:", typeof myCat);
    console.log("Je kočka instancí objektu 'Cat':", myCat instanceof Cat);
    console.log("Je kočka instancí objektu 'Object':", myCat instanceof Object);
    console.log("Je 'Object' instancí objektu kočka:", {} instanceof Cat);

    console.log("");

    console.log("Je kočka instancí objektu 'Animal':", myCat instanceof Animal);
    console.log("Je moje kočka savec:", myCat.isMammal());
    console.log("Počet nohou:", myCat.getNumberOfLegs());

    console.log("");


    var myAnimal = new Animal(false);
    console.log("Moje zvíře:", myAnimal);
    console.log("Je moje zvíře instance typu 'Cat':", myAnimal instanceof Cat);


   
}